/*
 * @author Walther Neuper, member of the ISAC-team, Copyright (c) 2005 by
 * Walther Neuper created Jun 10, 2005 10:44:45 AM Institute for
 * Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.bridge;

import isac.interfaces.ICalcIterator;
import isac.interfaces.IToCalc;
import isac.interfaces.IToUser;
import isac.kestore.KEStoreServices;
import isac.util.CalcChanged;
import isac.util.Formalization;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.CalcHead;
import isac.util.formulae.HierarchyKey;
import isac.util.formulae.Match;
import isac.util.formulae.Model;
import isac.util.formulae.ModelItem;
import isac.util.formulae.ModelItemList;
import isac.util.formulae.Position;
import isac.util.formulae.ProblemID;
import isac.util.parser.FormalizationDigest;
import isac.util.tactics.SimpleTactic;
import isac.util.tactics.StringListTactic;
import isac.util.tactics.Tactic;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Vector;

import junit.framework.TestCase;

/**
 * tests kernel in the specify-phase
 * 
 * @author Walther Neuper
 * @version 1.0
 */
public class TestModel extends TestCase {

    Formalization fmz_;

    MathEngine me_;

    /**
     * start the maximum-example
     * 
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_7a.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        fmz_ = fd.parseFormalization((String) xmlstr);

        MathEngine.init("localhost");
        me_ = MathEngine.getMathEngine();
    }

    /**
     * interactively complete the Model by using the <NEXT>button;; direct calls
     * of the math-engine
     */
    public void testAddItems() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestModel#testAddItems");

        //start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //fill the Models Items with Descriptions for userguidance:
        //tac = Model_Problem ...
        ch = ct.modelProblem();
        //WN050725: "incomplete \"fixedValues []\"]"
        assertEquals("[\"incomplete (fixedValues [])\"]", ch.getModel()
                .getGiven().toString());
        System.out.println(".end step_b-----");
        //end step_b ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //fill the Models Items stepwise with the <Next>-button
        ch = ct.nextSpecify();
        //WN050725: like above
        assertEquals("[\"correct (fixedValues [r = Arbfix])\"]", ch.getModel()
                .getGiven().toString());
        //and so on...
        ch = ct.nextSpecify();
        ch = ct.nextSpecify();
        //lists are filled elementwise:
        //WN050725: like above
        assertEquals("incomplete (valuesFor [a])", ch.getModel().getFind()
                .getItems().elementAt(1).toString());
        ch = ct.nextSpecify();
        //WN050725: like above
        assertEquals("correct (valuesFor [a, b])", ch.getModel().getFind()
                .getItems().elementAt(1).toString());
        ch = ct.nextSpecify();
        ch = ct.nextSpecify();
        System.out.println(ch.getModel().toString());
        //now the Model is complete; for completing the CalcHead by completing
        // the Specification see testEditItems
        System.out.println(".end step_c-----");
        //end step_c ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        System.out.println("---END isac.bridge.TestModel#testAddItems");
    }

    /**
     * {\bf\UC{Having \isac{} Complete the
     * CalcHead}\label{SPECIFY:COMPLETE:all}\\} The user requests a CalcHead,
     * containing some input, to be completed automatically. WN0506 What kind of
     * messed input should be overwritten/corredted ???
     * 
     * @see isac.functest.specify.TestModel#testCompleteCalcHead() for how this
     *      looks like on the CalcHeadPanel
     */
    public void testCompleteCalcHead() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestModel#testCompleteCalcHead");

        //start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the CalcHead with ONE SPECIAL BUTTON ...
        ch = ct.completeCalcHead();
        //the model is complete
        //WN050725: "correct \"fixedValues [r = Arbfix]\""
        assertEquals("correct (fixedValues [r = Arbfix])", ch.getModel()
                .getGiven().getItems().elementAt(0).toString());
        //WN050725 as above
        assertEquals("correct (maximum A)", ch.getModel().getFind().getItems()
                .elementAt(0).toString());
        //WN050725 as above
        assertEquals("correct (valuesFor [a, b])", ch.getModel().getFind()
                .getItems().elementAt(1).toString());
        //WN050725 as above
        assertEquals("correct (relations [A = a * b, "
                + "(a / 2) ^ 2 + (b / 2) ^ 2 = r ^ 2])", ch.getModel()
                .getRelate().getItems().elementAt(0).toString());
        //the Specification is complete
        assertEquals(ch.getSpecification().toSMLString(), "\"DiffApp.thy\","
                + "[\"maximum_of\",\"function\"],"
                + "[\"DiffApp\",\"max_by_calculus\"]");
        System.out.println(".end step_b-----");
        //end step_b
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        System.out.println("---END isac.bridge.TestModel#testCompleteCalcHead");
    }

    /**
     * complete the Model by hitting ONE SPECIFIC BUTTON. WN0506 What kind of
     * messed input should be overwritten/corredted ???
     * 
     * @see isac.functest.specify.TestModel#testCompleteModel() for how this
     *      looks like on the CalcHeadPanel
     */
    public void testCompleteModel() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestModel#testCompleteModel");

        //start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Model with ONE SPECIAL BUTTON ...
        ch = ct.completeModel();
        //WN050725: "correct \"fixedValues [r = Arbfix]\""
        assertEquals(ch.getModel().getGiven().getItems().elementAt(0)
                .toString(), "correct (fixedValues [r = Arbfix])");
        //WN050725 as above
        assertEquals("correct (maximum A)", ch.getModel().getFind().getItems()
                .elementAt(0).toString());
        assertEquals("correct (valuesFor [a, b])", ch.getModel().getFind()
                .getItems().elementAt(1).toString());
        assertEquals("correct (relations [A = a * b, "
                + "(a / 2) ^ 2 + (b / 2) ^ 2 = r ^ 2])", ch.getModel()
                .getRelate().getItems().elementAt(0).toString());
        System.out.println(".end step_b-----");
        //end step_b
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        System.out.println("---END isac.bridge.TestModel#testCompleteModel");
    }

    /**
     * complete the Model by manual input of the Items; any number of inputs;;
     * direct calls of the math-engine
     * 
     * @see isac.functest.specify.TestModel#testEditItems() for how this looks
     *      like on the CalcHeadPanel see smltest/FE-interface/interface.sml ---
     *      maximum-example
     */
    public void testEditItems() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestModel#testEditItems");

        //start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        Model model = null;
        ModelItemList itms = null;
        ModelItem itm = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //fill the CalcHead with Descriptions:
        ch = ct.modelProblem();
        String test = "\nModel:\n"//.........................................
                + "Given:\n"//.........................................
                + "[\"incomplete (fixedValues [])\"]\n"//...............
                + "Where:\n"//.........................................
                + "[]\n"//.........................................
                + "Find:\n"//.........................................
                + "[\"incomplete (maximum)\", \"incomplete (valuesFor)\"]\n"
                + "Relate:\n"//.........................................
                + "[\"incomplete (relations [])\"]";
        assertEquals(ch.getModel().toString(), test);

        //input of two items, 'fixedValues [r=Arbfix]' and 'maximum A'...
        model = new Model();
        itms = new ModelItemList();
        itm = new ModelItem();
        itm.setText("fixedValues [r=Arbfix]");
        itms.addItem(itm);//status not relevant at input
        model.setGiven(itms);
        itms = new ModelItemList();
        itm = new ModelItem();
        itm.setText("maximum A");
        itms.addItem(itm);
        // these are left unchanged from above ...
        itm = new ModelItem();
        itm.setText("valuesFor");
        itms.addItem(itm);
        model.setFind(itms);
        itms = new ModelItemList();
        itm = new ModelItem();
        itm.setText("relations []");
        itms.addItem(itm);
        model.setRelate(itms);

        ch.setModel(model);
        ch = ct.checkCalcHead(ch);
        test = "\nModel:\n"//.........................................
                + "Given:\n"//.........................................
                + "[\"correct (fixedValues [r = Arbfix])\"]\n"//.........
                + "Where:\n"//.........................................
                + "[]\n"//.........................................
                + "Find:\n"//.........................................
                + "[\"correct (maximum A)\", \"incomplete (valuesFor)\"]\n"
                + "Relate:\n"//.........................................
                + "[\"incomplete (relations [])\"]";
        System.out.println(".ch = " + ch.toString());
        assertEquals(ch.getModel().toString(),test);

        System.out.println("---END isac.bridge.TestModel#testEditItems");
    }

    /**
     * {\bf\UC{Having \isac{} Fill One Field of the Model with a correct
     * content}\label{SPECIFY:COMPLETE:modelone}\\}
     * 
     * @see isac.functest.specify.TestModel#testFillOneItem() for how this looks
     *      like on the CalcHeadPanel
     * 
     * related SML is not implemented; would also require the bridge to send the
     * Items' status from Java to SML
     */
    public void testFillOneItem() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestModel#testFillOneItem");
        System.out.println("TODO");
        assertEquals("TODO", "TODO");
        System.out.println("---END isac.bridge.TestModel#testFillOneItem");
    }

}